% factorization_ct
% description: The main function for factorization method
%
% Inputs:
% rpy_ix: x coordinate of feature points in images [no. of feature, no. of images] = size(rpy_ix)
% rpy_iy: y coordinate of feature points in images
% flen: focal length in no. of pixels
% init_center: initial center point of the 3D model
% noit: no. of iteration
%
% Outputs:
% P: found model
% RT_sol: Translation [1:3] and Rotation angle [4:6]
% error_trucco_all_time: (not implemented)
% mse_error_model_pixels2_loop: (not implemented)
function [P,RT_sol,error_trucco_all_time,mse_error_model_pixels2_loop]...
    = cv_factorization(ixy_filename,sfm_method)
load (ixy_filename)
init_center=[0 0 zinit];
noit=50;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[nf,Nimg] = size(rpy_ix);
RT_sol = zeros(6,Nimg);
% xx=rpy_ix-repmat(rpy_ix(:,1),1,Nimg);
% yy=rpy_iy-repmat(rpy_iy(:,1),1,Nimg);
%
% orthographic_dx=mean(xx);
% orthographic_dy=mean(yy);

[NN,TT]=size(rpy_ix)

nn=0
for n=1:NN
    if isempty( find(rpy_ix(n,:) < -9000))
        nn=nn+1;
        temp_ix(round(nn),:)=rpy_ix(n,:);
        temp_iy(round(nn),:)=rpy_iy(n,:);
    end
end
rpy_ix=temp_ix;
rpy_iy=temp_iy;
NN=nn;

xx=rpy_ix-repmat(rpy_ix(:,1),1,Nimg);
yy=rpy_iy-repmat(rpy_iy(:,1),1,Nimg);
orthographic_dx=mean(xx);
orthographic_dy=mean(yy);

%ryp_ix=iix;
%ryp_iy=iiy;
switch sfm_method
    case 'original_factorization'
        [P,RT_sol,error_trucco_all_time,mse_error_model_pixels2_loop]...
            = factorization_ct(rpy_ix,rpy_iy,flen,init_center,noit)

    case 'sequential_factorization'
        [P,RT_sol,error_trucco_all_time,mse_error_model_pixels2_loop]...
            = sequentialfac_ct(rpy_ix,rpy_iy,flen,init_center,noit)

    case 'paraperspective_factorization'
        [P,RT_sol,error_trucco_all_time,mse_error_model_pixels2_loop]...
            = paraperspectivefac_ct(rpy_ix,rpy_iy,flen,init_center,init_center,noit)
end

%plot ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
figure(561)
clf
hold on
subplot(3,1,1),plot(RT_sol(4,:)*180/pi,'r-');ylabel('an1');
subplot(3,1,2),plot(RT_sol(5,:)*180/pi,'r-');ylabel('an2');
subplot(3,1,3),plot(RT_sol(6,:)*180/pi,'r-');ylabel('an3');
xlabel('orginal factorization method , rotation angles vs time')

figure(562)
clf
hold on
subplot(2,1,1),plot(orthographic_dx,'r-');ylabel('trans-X');
subplot(2,1,2),plot(orthographic_dy,'r-');ylabel('trans-Y');
%subplot(3,1,3),plot(RT_sol(3,:),'r-');ylabel('trans-Z');
xlabel('orthograhic transaltion vs time')

figure(563)
clf
hold on
plot3(P(1,:),P(2,:),P(3,:),'r.');
sfm_method
title('model found by factorization')

error_trucco_all_time = zeros(noit,1);
mse_error_model_pixels2_loop = zeros(noit,1);

model=P;
ix=rpy_ix;
iy=rpy_iy;

save factorization_out
return

