%1-pass algo, init model using lowe202
function [model,rt,er,et]=lowe101(iter,lowe101_infile,lowe101_outfile,display_tag)

if nargin<4, display_tag=0; end
if nargin<3, lowe101_outfile='lowe101_out'; end
if nargin<2, lowe101_infile='syn_dat'; end
if nargin<1, iter=10; end

load(lowe101_infile);

if ~exist('model')      %if model not exists, init
%    function [model,rt,er,et]=lowe202(iter, lowe202_lowe101_infile, lowe202_lowe101_outfile,syn_tag,syn_model,display_tag)
    [model,rt]=lowe202(1, lowe101_infile, lowe101_outfile, display_tag);
end

t0=cputime;

[N,T]=size(ix);
R=[];
D=[];
for i=1:T           %break rt into R and D
    R = [R rt(:,1:3,i)];
    D = [D rt(:,4,i)];
    
end

lambda = 1; %lambda = 1 %is ok; %lambda = 0.001 %is orginal in backproj1
eye_mat = speye(3*N+6*T);

check_terminate_tag=0;  %1 the number of iterations may be smaller; 0 n-->iter

for n=1:iter
   
    pause(0)
    'lowe101'
    n
    [e, J]=my_fun(model,ix,iy,R,D,flen);
    %spy(J);
    old_err=mserror(e);%-------------------display-----------
    Je = J'*e;  
    
    JJ = J'*J;
    %spy(JJ)
    LM = lambda*eye_mat;
    JJ = JJ+JJ.*LM;    %add the Levenberg-Marquardt term             
    dx=JJ\Je;
    dM = reshape(dx(1:3*N),3,N);
    dRT = reshape(dx(3*N+1:end),6,T);
    
    %     %no LM section
    %     model = model + dM;
    %     D = D + dRT(1:3,:);
    %     for t=1:T
    %         R(:,(t-1)*3+1:(t-1)*3+3) = rpyMat(dRT(4:6,t))*R(:,(t-1)*3+1:(t-1)*3+3);  %improve the angle
    %     end
    
    %LM section
    tmodel = model + dM;
    tD = D + dRT(1:3,:);
    for t=1:T
        tR(:,(t-1)*3+1:(t-1)*3+3) = rpyMat(dRT(4:6,t))*R(:,(t-1)*3+1:(t-1)*3+3);  %improve the angle
    end
    e = my_fun(tmodel,ix,iy,tR,tD,flen);
    new_err=mserror(e); 
    if new_err <= old_err
        err = new_err;      
        model = tmodel;
        D = tD;
        R = tR;
        lambda = lambda/2;
        if check_terminate_tag==1 & (old_err-new_err)/old_err < 0.001  %too slow
            break;
        end
    else
        err = old_err;
        lambda = lambda*2;
    end

    er(n)=err  %2d error
    
    if n==1 %will give  a better look to fill up the first data
        et(n)=0;
    else
        et(n)=cputime-t0;
    end

end
%----------------------------------------------------------------------
rt=zeros(3,4,T);
for i=1:T
    rt(:,1:3,i)=R(:,3*(i-1)+1:3*i);
    rt(:,4,i)=D(:,i);
end

%generate all views of  model
[N,T]=size(ix);
model=[model;ones(1,N)];
m=cell(T,2);
for t=1:T
    v=find(~isnan(ix(:,t)));
    X=rt(:,:,t)*model(:,v);   %rt=3*4*frames
    m{t,1}=t;
    m{t,2}=X;
end

model=model(1:3,:);     %eliminate the 4th row of ones
save(lowe101_outfile,'m','ix','iy','flen','zinit','model','rt','er','et');        

R=zeros(3,3,T);     %compute the rotated rodrigue angles
for t=1:T
    R(:,:,t)=rt(1:3,1:3,t);
end
angles=rotate_angles(R);
angles(1:5)
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%% functions  %%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function e = mserror(v)  %calc the mean square error
e = sqrt(sum(v.^2)/(length(v)/2));

function [e,J] = my_fun(model,ix,iy,R,D,F)
[N,T] = size(ix);
Tbase=0;  
N3=3*N;  
% T6=6*T;
% MA = 2*N*T;
% NA = 3*N+6*T;
J=[]; e=[]; i=[]; j=[]; k=[];
for t=1:T			%build Jacobian matrix
    v = find(~isnan(ix(:,t)));         %find valid points
    M = length(v);
    rt = R(:, (t-1)*3+1:3*t);     %3*3 matrix
    Mr = rt*model(:,v);
    P = Mr + repmat(D(:,t),1,M);
    X=P(1,:);
    Y=P(2,:);
    Z=P(3,:);
    Xr=Mr(1,:);
    Yr=Mr(2,:);
    Zr=Mr(3,:);
    ZZ=Z.*Z;
    x = F*X./Z;
    y = F*Y./Z;
    
    e = [e; ix(v,t)-x'; iy(v,t)-y'];
    
    if nargout > 1 
        %ti = [(t-1)*N2+1 : (t-1)*N2+N]';
        ti = [Tbase+1 : Tbase+M]';
        Tbase = Tbase+M;
        
        i = [i; ti];            %dxdU
        tmp=[1:3:N3]';
        j = [j; tmp(v)];
        dxdU = F*(R(1,(t-1)*3+1)./Z - R(3,(t-1)*3+1).*X./ZZ);
        k=[k;dxdU'];
        
        i = [i; ti];            %dxdV
        tmp=[2:3:N3]';
        j = [j; tmp(v)];
        dxdV = F*(R(1,(t-1)*3+2)./Z - R(3,(t-1)*3+2).*X./ZZ);
        k=[k;dxdV'];
        
        i = [i; ti];            %dxdW  
        tmp=[3:3:N3]';
        j = [j; tmp(v)];
        dxdW = F*(R(1,(t-1)*3+3)./Z - R(3,(t-1)*3+3).*X./ZZ);
        k=[k;dxdW'];
        
        i = [i; ti];            %dxdt1
        j = [j; repmat(N3+(t-1)*6+1,M,1)];
        dxdt1 = F*(1./Z);
        k=[k;dxdt1'];
        %dxdt2=0; ignore
        i = [i; ti];            %dxdt3
        j = [j; repmat(N3+(t-1)*6+3,M,1)];
        dxdt3 = -F.*X./ZZ;
        k=[k;dxdt3'];
        
        i = [i; ti];            %dxda1
        j = [j; repmat(N3+(t-1)*6+4,M,1)];
        dxda1 = -F.*X.*Yr./ZZ;
        k=[k;dxda1'];
        
        i = [i; ti];            %dxda2
        j = [j; repmat(N3+(t-1)*6+5,M,1)];
        dxda2 = F*(X.*Xr+Z.*Zr)./ZZ;
        k=[k;dxda2'];
        
        i = [i; ti];            %dxda3
        j = [j; repmat(N3+(t-1)*6+6,M,1)];
        dxda3 = -F.*Yr./Z;
        k=[k;dxda3'];
        
        %process iy points
        ti = [Tbase+1 : Tbase+M]';
        Tbase = Tbase+M;
        
        i = [i; ti];            %dydU
        tmp=[1:3:N3]';
        j = [j; tmp(v)];
        dydU = F.*(R(2,(t-1)*3+1)./Z - R(3,(t-1)*3+1).*Y./ZZ);
        k=[k;dydU'];
        
        i = [i; ti];            %dydV
        tmp=[2:3:N3]';
        j = [j; tmp(v)];
        dydV = F.*(R(2,(t-1)*3+2)./Z - R(3,(t-1)*3+2).*Y./ZZ);
        k=[k;dydV'];
        
        i = [i; ti];            %dydW
        tmp=[3:3:N3]';
        j = [j; tmp(v)];
        dydW = F.*(R(2,(t-1)*3+3)./Z - R(3,(t-1)*3+3).*Y./ZZ);
        k=[k;dydW'];
        
        %dydt1=0, ignored
        
        i = [i; ti];            %dydt2
        j = [j; repmat(N3+(t-1)*6+2,M,1)];
        dydt2 = F.*(1./Z);
        k=[k;dydt2'];
        
        i = [i; ti];            %dydt3
        j = [j; repmat(N3+(t-1)*6+3,M,1)];
        dydt3 = -F.*Y./ZZ;
        k=[k;dydt3'];
        
        i = [i; ti];            %dyda1
        j = [j; repmat(N3+(t-1)*6+4,M,1)];
        dyda1 = -F.*(Y.*Yr+Z.*Zr)./ZZ;   
        k=[k;dyda1'];
        
        i = [i; ti];            %dyda2
        j = [j; repmat(N3+(t-1)*6+5,M,1)];
        dyda2 = F.*Xr.*Y./ZZ;
        k=[k;dyda2'];
        
        i = [i; ti];            %dyda3
        j = [j; repmat(N3+(t-1)*6+6,M,1)];
        dyda3 = F.*Xr./Z;
        k=[k;dyda3'];
    end
end
J = sparse(i,j,k);
return

function R = rpyMat (angs)

% Return the 3x3 rotation matrix described by a set of Roll, Pitch and Yaw
% angles.

cosA = cos (angs(3));
sinA = sin (angs(3));
cosB = cos (angs(2));
sinB = sin (angs(2));
cosC = cos (angs(1));
sinC = sin (angs(1));

cosAsinB = cosA .* sinB;
sinAsinB = sinA .* sinB;

R = [ cosA.*cosB  cosAsinB.*sinC-sinA.*cosC  cosAsinB.*cosC+sinA.*sinC ;
    sinA.*cosB  sinAsinB.*sinC+cosA.*cosC  sinAsinB.*cosC-cosA.*sinC ;
    -sinB            cosB.*sinC                 cosB.*cosC         ];
