%lowe212: lowe202+NaN
function [model,rt,er,et,ix,iy]=lowe212(iter, infile, outfile, syn_model,display_tag)

%infile is the ix,iy file with NaN 
if nargin<5, display_tag=0; end
if nargin<4, syn_model=[]; end
if nargin<3, outfile='lowe212_out'; end
if nargin<2, infile='syn_dat'; end
if nargin<1, iter=6; end

load(infile);

if ~exist('model')  %if model not exists in infile,  init model

    [model,rt,ix,iy]=cv_initmodel(infile,'temp',20);
%     model = zeros(3,N);   %model = 3*N
%     model = [ix(:,1)'; iy(:,1)'; zeros(1,N)]; %back project the first image
%     model = (zinit/flen) .* model;
end
model = [model; ones(size(model(1,:)))];     %turn into homogeneous coordinate

%err=project_err(flen, model, ix, iy, rt,0);
%original_err=mserror(err)

[N,T]=size(ix);
if ~exist('rt')
    rt=zeros(3,4,T);
    rt(:,:,1)=[eye(3) [0;0;zinit]];
end

if size(rt,3)~=T
    disp('lowe212: size of rt not matched with ix,iy');
end

t0=cputime;   %reset counter
et=[]; er=[];
for n = 1:iter

    pause(0)
    'lowe212'
    n
    
    for j=1:T
        if rt(3,4,j)==0
            rt(:,:,j)=rt(:,:,j-1);
        end
        v = find(~isnan(ix(:,j)));         %find valid points
        err=column_err(flen, model(:,v), ix(v,j), iy(v,j), rt(:,:,j),0);
        
        J = JacobRT(flen, model(:,v), rt(:,:,j));      
        
        delta = J\err;		%delta=3T+3T
        dT = delta(1:3);
        dR = delta(4:6);
        rt(1:3,4,j) = rt(1:3,4,j)+dT;
        rt(1:3,1:3,j) = rpyMat(dR)*rt(1:3,1:3,j);
        
        %added by khw
%        constraint = rpyAng(rt(1:3,1:3,1));   %get angle of 1st image
%        rt(1:3,1:3,j)=rpymat(  rpyAng(rt(1:3,1:3,j))-constraint);
        %adding ends
    end
    
    err=project_err(flen, model, ix, iy, rt,0);         
    err2=mserror(err);
    
    for i=1:N
        if model(4,i)~=0
            v = find(~isnan(ix(i,:)));
            err=row_err(flen,model(:,i),ix(i,v),iy(i,v),rt(:,:,v));
            J = JacobM(flen, model(:,i), rt(:,:,v));
            delta = J\err;
            model(1:3,i) = model(1:3,i) + delta;
        end
    end
        
    err=project_err(flen, model, ix, iy, rt,0);
    er(n)=mserror(err)  %2d error
    if n==1
        et(n)=0;
    else
        et(n)=cputime-t0;
    end
%if abs((err3-err1)/err1) < 0.001  %if improvement is less than 0.1%
    %    break
    %end
end 
%for display result for the final result
project_err(flen, model, ix, iy, rt, display_tag);

%generate all views of  model
[N,T]=size(ix);
m=cell(T,2);
for t=1:T
    v=find(~isnan(ix(:,t)));
    X=rt(:,:,t)*model(:,v);   %rt=3*4*frames
    m{t,1}=t;
    m{t,2}=X;
end

angles=rotate_angles(rt(:,1:3,:))           %check estimated angles

model=model(1:3,:);     %eliminate the 4th row of ones
if ~exist('syn_trans_angles')       %if syn_trans_angles not exist, create a dummy one
    syn_trans_angles=[];
end
save(outfile,'m','ix','iy','flen','model','rt','er','et','syn_model','syn_trans_angles','zinit');        

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure(231)
% clf
% plot(et,er,'r+-')
% plot_title('lowe212, 2d er plot vs cpu_time');
% %et
% 
% figure(30)
% hold on
% plot(et,er,'r+-')
% %plot_title('lowe212, 2d er plot vs cpu_time');
% %et



return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%              fucntions                                 %%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function e = mserror(v)  %calc the mean square error
e = sqrt(sum(v.^2)/(length(v)/2));
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function err=column_err(flen,model,ix,iy,rt,display_tag,t)
%return err, a column vector of max size 3*N*T
[N,T]=size(ix);
if T~=1
    error('column_err: ix, iy must be a column vector');
end
X = rt*model;
x = flen.*X(1,:)./X(3,:);
y = flen.*X(2,:)./X(3,:);
ex = ix - x';
ey = iy - y';
err = [ex; ey]; %ix is 300x1 ,

if display_tag==1 
    t
    figure(412)
    clf
    hold on
    plot(ix,iy,'bx');
    plot(x',y','ro');
    plot_title('in cv_lowe212.m: col_err,2d error: syn_dat=bx,found=ro, press for next frame' );
    pause
end

return



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function err=row_err(flen,model,ix,iy,rt)
%return err, a column vector of max size 3*N*T
[N,T]=size(ix);
if N~=1
    error('row_err: ix, iy must be a row vector');
end
err=zeros(2*T,1);
for t=1:T
    X = rt(:,:,t)*model;
    x = flen.*X(1,:)./X(3,:);
    y = flen.*X(2,:)./X(3,:);
    err(t) = (ix(1,t) - x)';
    err(T+t) = (iy(1,t) - y)';
end
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function err=project_err(flen,model,ix,iy,rt,display_tag)
%return err, a column vector of max size 3*N*T
[N,T]=size(ix);
err=[];

%hold on
for t=1:T
    v = find(~isnan(ix(:,t)));         %find valid points that is not the start point
    s = find(model(4,:)==0);        %find uninitialised points
    v = setdiff(v,s);               %eliminate uninit points
    err = [err; column_err(flen,model(:,v),ix(v,t),iy(v,t),rt(:,:,t),display_tag,t)];
end


return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function J=JacobM(flen, model, rt)
N = size(model,2);    %model=3,N
if N~=1
    error('JacobM: model size must be 4*1');
end
T = size(rt,3);       %rt=3,4,T
J=zeros(2*T,3);

for t=1:T             %index T vertical blocks
    V = rt(:,:,t)*model;
    X = V(1,:);
    Y = V(2,:);
    Z = V(3,:);
    Z2 = Z.*Z;
    XZ2 = X./Z2;
    YZ2 = Y./Z2;
    a11 = rt(1,1,t)./Z - rt(3,1,t).*XZ2;
    a12 = rt(1,2,t)./Z - rt(3,2,t).*XZ2;
    a13 = rt(1,3,t)./Z - rt(3,3,t).*XZ2;
    a21 = rt(2,1,t)./Z - rt(3,1,t).*YZ2;
    a22 = rt(2,2,t)./Z - rt(3,2,t).*YZ2;
    a23 = rt(2,3,t)./Z - rt(3,3,t).*YZ2;
    a1 = [a11' a12' a13'];
    a2 = [a21' a22' a23'];
    
    J(t,:) = a1;
    J(T+t,:) = a2;
end
J = flen.*J;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%extended lowe
function J=JacobRT(flen, model, rt)
%find RT with the R in frame 1 included
T = size(rt,3);       %rt=3,4,T
if (T~=1)
    error('rt size must be 3*4*1 only (T must equal to 1');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%this is to fix the bug in the xlowe implementation--begin
N = size(model,2);
J=zeros(2*N,6);	 
XXR = rt*model;		%X=RM+T
J=zeros(2*N,6);%setup the matrix
for j=1:N 
    XR=XXR(1,j)
    YR=XXR(2,j)
    ZR=XXR(3,j)
    ZR2=XXR(3,j).*XXR(3,j)
    
    X=model(1,j)
    Y=model(2,j)
    Z=model(3,j)
    J(2*(j-1)+1,:)=flen*[1/ZR 0    -XR/ZR2   -(Y*XR)/ZR2     (X*XR+Z*ZR)/ZR2  -Y/ZR];
    J(2*(j-1)+2,:)=flen*[0    1/ZR -YR/ZR2  -(Y*YR+Z*ZR)/ZR2 (XR*X)/ZR2      X/ZR];
end
%this is to fix the bug in the xlowe implementation--ends


%buggy begin----xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
% %1:3T columns for translation
% X = rt*model;		%X=RM+T
% XR = rt(1:3,1:3)*model(1:3,:); %XR=RM
% Z2 = X(3,:).*X(3,:);
% 
% %translation part 
% x1 = 1./X(3,:);
% x2 = zeros(1,N);
% x3 = -X(1,:)./Z2;
% y1 = x2;
% y2 = x1;
% y3 = -X(2,:)./Z2;
% J(:,1:3) = [x1' x2' x3'; y1' y2' y3'];
% 
% %rotation part
% %x1 = -X(1,:).*XR(2,:)./Z2;       %dxdalpha, orgional
% x1 = -X(2,:).*XR(1,:)./Z2;       %dxdalpha, fix khwong 040412
% x2 = (X(1,:).*XR(1,:)+X(3,:).*XR(3,:))./Z2;
% x3 = -XR(2,:)./X(3,:);
% y1 = -(X(2,:).*XR(2,:)+X(3,:).*XR(3,:))./Z2;
% y2 = (XR(1,:).*X(2,:))./Z2;
% y3 = XR(1,:)./X(3,:);
% J(:,4:6) = [x1' x2' x3'; y1' y2' y3'];
% 
% J = flen.*J;
% buggy ends XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXx


% Author: Rodrigo Carceroni
% Disclaimer: This code comes with no guarantee at all and its author
%   is not liable for any damage that its utilization may cause.

function R = rpyMat (angs)

% Return the 3x3 rotation matrix described by a set of Roll, Pitch and Yaw
% angles.

cosA = cos (angs(3));
sinA = sin (angs(3));
cosB = cos (angs(2));
sinB = sin (angs(2));
cosC = cos (angs(1));
sinC = sin (angs(1));

cosAsinB = cosA .* sinB;
sinAsinB = sinA .* sinB;

R = [ cosA.*cosB  cosAsinB.*sinC-sinA.*cosC  cosAsinB.*cosC+sinA.*sinC ;
    sinA.*cosB  sinAsinB.*sinC+cosA.*cosC  sinAsinB.*cosC-cosA.*sinC ;
    -sinB            cosB.*sinC                 cosB.*cosC         ];

function angles = rotate_angles(R)
T = size(R,3);      %R=3*3*T
angles=zeros(1,T);
for t=1:T
    %Rn=r*R1 => r=Rn*R1'
    r = R(:,:,t)*R(:,:,1)';
    A = 0.5*(r-r');
    a = 0.5*sum(sum(A.^2));
    angles(t) = asin(sqrt(a));
end
