function lowe_filter(infile,outfile,initfile)

if nargin<3, initfile=[]; end
if nargin<2, outfile='syn_dat'; end
if nargin<1, infile='lowe202_out'; end

load(infile);

[N,T]=size(ix);
model=[model;ones(1,N)];  %into homogeneous coordinate

e=project_err(flen, model, ix, iy, rt);
npoints = length(e)/2;

sd = sqrt(sum(e.^2)/npoints);     %mean square error from 0
limit=2*sd;         %set cut-off point=5*sd
limit2 = limit^2;   %so that err needs not be sqrt

for t=1:T           %eliminate err>limit
    v = find(~isnan(ix(:,t)));
    e = column_err(flen,model(:,v),ix(v,t),iy(v,t),rt(:,:,t));
    ex = e(1:length(v));
    ey = e(length(v)+1:end);
    e2 = ex.^2 + ey.^2;        %square error
    d = find(e2>limit2);
    dv = v(d);      %dv is a list of index to ix(:,t)
    ix(dv,t) = NaN; %mark large error as invalid point
    iy(dv,t) = NaN;
end

%model=model(1:3,:);
if ~isempty(initfile)
    load(initfile,'model','rt');  %replace current model by the initfile's model
end
model=model(1:3,:);

syn_mode_model=0
if syn_mode_model==1
save(outfile,'model','rt','syn_model','syn_trans_angles','ix','iy','flen','m','et','er','zinit');        
else
save(outfile,'model','rt','ix','iy','flen','m','et','er','zinit');        
end

function err=project_err(flen,model,ix,iy,rt)
[N,T]=size(ix);
err=[];
for t=1:T
    v = find(~isnan(ix(:,t)));         %find valid points that is not the start point
    err = [err; column_err(flen,model(:,v),ix(v,t),iy(v,t),rt(:,:,t))];
end

function err=column_err(flen,model,ix,iy,rt)
[N,T]=size(ix);
if T~=1
    error('column_err: ix, iy must be a column vector');
end
X = rt*model;
x = flen.*X(1,:)./X(3,:);
y = flen.*X(2,:)./X(3,:);
ex = ix - x';
ey = iy - y';
err = [ex; ey];
