% factorization_ct
% description: The main function for factorization method
%
% Inputs:
% rpy_ix: x coordinate of feature points in images [no. of feature, no. of images] = size(rpy_ix)
% rpy_iy: y coordinate of feature points in images
% flen: focal length in no. of pixels
% init_center: initial center point of the 3D model
% noit: no. of iteration
%
% Outputs:
% P: found model
% RT_sol: Translation [1:3] and Rotation angle [4:6]
% error_trucco_all_time: (not implemented)
% mse_error_model_pixels2_loop: (not implemented)
% function [P,RT_sol,error_trucco_all_time,mse_error_model_pixels2_loop]...
%     = factorization_2cam(rpy_ix,rpy_iy,flen,init_center,noit)
function [P,RT_sol,error_trucco_all_time,mse_error_model_pixels2_loop]=factorization_2cam
toolpaths=sub_add_path;

load ba_inpar
init_center=in.init_center
flen=cam.flen
%(rpy_ix,rpy_iy,flen,init_center,noit)
load real_dat
noit=50
K=2
RK=RK(:,1:3,1:K);
rpy_ix1=xycell{1,1};
rpy_iy1=xycell{1,2};
rpy_ix2=xycell{2,1};
rpy_iy2=xycell{2,2};

[xx1,yy1]=cv_remove_nan_features(rpy_ix1,rpy_iy1);
[xx2,yy2]=cv_remove_nan_features(rpy_ix2,rpy_iy2);

[nfeats_1,TT]=size(xx1)
[nfeats_2,TT]=size(xx2)
%%%%%%%%%%%%%%%%%%%%%%%%
%cam1
[nf1,Nimg] = size(xx1)
xxx1=xx1-repmat(xx1(:,1),1,Nimg);
yyy1=yy1-repmat(yy1(:,1),1,Nimg);
orthographic_dx1=mean(xxx1);
orthographic_dy1=mean(yyy1);
%cam2
[nf2,Nimg] = size(xx2)
xxx2=xx2-repmat(xx2(:,1),1,Nimg);
yyy2=yy2-repmat(yy2(:,1),1,Nimg);
orthographic_dx2=mean(xxx2);
orthographic_dy2=mean(yyy2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%pause(0)
% one cam
cam_selected='12', %1,2,12
switch cam_selected
    case '1'
        rpy_ix=[xx1]; %263
        rpy_iy=[yy1];
    case '2'
        rpy_ix=[xx2]; %345
        rpy_iy=[yy2];
    case '12'
        rpy_ix=[xx1;xx2];
        rpy_iy=[yy1;yy2];
end


pause(0)

[nf,Nimg] = size(rpy_ix);

% find X and Y for factorization W
%X = rpy_ix' * init_center(3) / flen;
%Y = rpy_iy' * init_center(3) / flen;
X = rpy_ix' * init_center(3) / flen;
Y = rpy_iy' * init_center(3) / flen;

% invoke factorization method
[R,S] = factorization(X,Y,noit);

% find the Direction of first Cam (respect to object centroid)
oi = R(1,:)';
oj = R(Nimg+1,:)';
ok=cross(oi,oj);
firstCamDirection = [oi oj ok]';

P = firstCamDirection*S;

% Rotation and translation (redundant for original factorization)
RT_sol = zeros(6,Nimg);
RT_sol(1:3,1) = init_center;
for i=1:Nimg
    oi = R(i,:)';
    oj = R(Nimg+i,:)';
    ok = cross(oi,oj);
    rotation_matrix = inv(firstCamDirection) * [oi oj ok]';
    RT_sol(4:6,i) = rpyAng(rotation_matrix);
end

error_trucco_all_time = zeros(noit,1);
mse_error_model_pixels2_loop = zeros(noit,1);




%plot ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
figure(560)
clf
plot(X(1,:),Y(1,:),'r.')
title('x,y')

figure(561)
clf
hold on

subplot(3,1,1),plot(RT_sol(4,:)*180/pi,'r-');ylabel('an1');
subplot(3,1,2),plot(RT_sol(5,:)*180/pi,'r-');ylabel('an2');
subplot(3,1,3),plot(RT_sol(6,:)*180/pi,'r-');ylabel('an3');


xlabel('orginal factorization method , rotation angles vs time')

figure(562)
clf
hold on
switch cam_selected
    case '12'
        subplot(3,1,1),plot(orthographic_dx1,'r-');ylabel('trans-X');
        subplot(3,1,2),plot(orthographic_dy1,'r-');ylabel('trans-Y');
        %x of 2nd camera is z in world reference (cam1)
        subplot(3,1,3),plot(orthographic_dx2,'r-');ylabel('trans-Z');
        %subplot(3,1,3),plot(RT_sol(3,:),'r-');ylabel('trans-Z');
        xlabel('cam1  and 2 translation vs time')
    case '1'
        subplot(3,1,1),plot(orthographic_dx1,'r-');ylabel('trans-X');
        subplot(3,1,2),plot(orthographic_dy1,'r-');ylabel('trans-Y');
        xlabel('cam1 orthograhic translation vs time')
    case '2'
        subplot(3,1,1),plot(orthographic_dx2,'r-');ylabel('trans-X');
        subplot(3,1,2),plot(orthographic_dy2,'r-');ylabel('trans-Y');
        xlabel('cam2 orthograhic translation vs time')
end

figure(563)
clf
hold on
P1=P(:,1:nfeats_1);
plot3(P1(1,:),P1(2,:),P1(3,:),'r.');

P2=P(:,nfeats_1+1:nfeats_1+nfeats_2);
adjusted_P2=RK(:,:,2)*P2;
plot3(adjusted_P2(1,:),adjusted_P2(2,:),adjusted_P2(3,:),'b.');

sfm_method
title('p1=r., p2=b+. model found by factorization')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
error_trucco_all_time = zeros(noit,1);
mse_error_model_pixels2_loop = zeros(noit,1);

model=P;
ix=rpy_ix;
iy=rpy_iy;

save factorization_out
return













sub_remove_path(toolpaths);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%           FUNCTIONS
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% factorization (Original FM, using Newton's method and least square method)
% X,Y
%
% R: camera reference geometry
% S: position of 3D points with respect to centriod of the points
function [R,S] = factorization(X,Y,noit)

% N: No of images
% n: No of feature pts
[N,n] = size(X);

% generate ~W
W = zeros(2*N,n);
W(1:N,:) = X;
W(N+1:2*N,:) = Y;
s_W = (sum(W'))' / n;
for i = 1:n
    W(:,i) = W(:,i) - s_W;
end

% svd
[U,D,V] = svds(W,3);
D2 = sqrtm(D);
RH = U*D2;
SH = D2*V';

Q = sub_factorization_newton(RH,noit);

R = RH * Q;
S = inv(Q) * SH;

function Q = sub_factorization_newton(RH,noit)
Q0 = [1 0 0;0 1 0;0 0 1];   %initial guess

[N,temp] = size(RH);
N = N / 2;
g = zeros(3*N,1);
rh_i = zeros(3,1);
rh_j = zeros(3,1);

for i = 1:noit
    % df = f' - f = g
    QQt = Q0 * Q0';
    for j = 1:N
        rh_i = (RH(j,:))';
        rh_j = (RH(j+N,:))';
        g(3*j-2) = rh_i' * QQt * rh_i - 1;
        g(3*j-1) = rh_j' * QQt * rh_j - 1;
        g(3*j) = rh_i' * QQt * rh_j;
    end

    % partial derivatives
    pfq11 = [2*Q0(1,1) Q0(2,1) Q0(3,1); Q0(2,1) 0 0; Q0(3,1) 0 0];
    pfq12 = [2*Q0(1,2) Q0(2,2) Q0(3,2); Q0(2,2) 0 0; Q0(3,2) 0 0];
    pfq13 = [2*Q0(1,3) Q0(2,3) Q0(3,3); Q0(2,3) 0 0; Q0(3,3) 0 0];
    pfq21 = [0 Q0(1,1) 0; Q0(1,1) 2*Q0(2,1) Q0(3,1); 0 Q0(3,1) 0];
    pfq22 = [0 Q0(1,2) 0; Q0(1,2) 2*Q0(2,2) Q0(3,2); 0 Q0(3,2) 0];
    pfq23 = [0 Q0(1,3) 0; Q0(1,3) 2*Q0(2,3) Q0(3,3); 0 Q0(3,3) 0];
    pfq31 = [0 0 Q0(1,1); 0 0 Q0(2,1); Q0(1,1) Q0(2,1) 2*Q0(3,1)];
    pfq32 = [0 0 Q0(1,2); 0 0 Q0(2,2); Q0(1,2) Q0(2,2) 2*Q0(3,2)];
    pfq33 = [0 0 Q0(1,3); 0 0 Q0(2,3); Q0(1,3) Q0(2,3) 2*Q0(3,3)];

    % Ax = b, A
    A = zeros(3*N,9);
    for j = 1:N
        rh_i = (RH(j,:))';
        rh_j = (RH(j+N,:))';

        A(3*j-2,1) = rh_i' * pfq11 * rh_i;
        A(3*j-2,2) = rh_i' * pfq12 * rh_i;
        A(3*j-2,3) = rh_i' * pfq13 * rh_i;
        A(3*j-2,4) = rh_i' * pfq21 * rh_i;
        A(3*j-2,5) = rh_i' * pfq22 * rh_i;
        A(3*j-2,6) = rh_i' * pfq23 * rh_i;
        A(3*j-2,7) = rh_i' * pfq31 * rh_i;
        A(3*j-2,8) = rh_i' * pfq32 * rh_i;
        A(3*j-2,9) = rh_i' * pfq33 * rh_i;

        A(3*j-1,1) = rh_j' * pfq11 * rh_j;
        A(3*j-1,2) = rh_j' * pfq12 * rh_j;
        A(3*j-1,3) = rh_j' * pfq13 * rh_j;
        A(3*j-1,4) = rh_j' * pfq21 * rh_j;
        A(3*j-1,5) = rh_j' * pfq22 * rh_j;
        A(3*j-1,6) = rh_j' * pfq23 * rh_j;
        A(3*j-1,7) = rh_j' * pfq31 * rh_j;
        A(3*j-1,8) = rh_j' * pfq32 * rh_j;
        A(3*j-1,9) = rh_j' * pfq33 * rh_j;

        A(3*j,1) = rh_i' * pfq11 * rh_j;
        A(3*j,2) = rh_i' * pfq12 * rh_j;
        A(3*j,3) = rh_i' * pfq13 * rh_j;
        A(3*j,4) = rh_i' * pfq21 * rh_j;
        A(3*j,5) = rh_i' * pfq22 * rh_j;
        A(3*j,6) = rh_i' * pfq23 * rh_j;
        A(3*j,7) = rh_i' * pfq31 * rh_j;
        A(3*j,8) = rh_i' * pfq32 * rh_j;
        A(3*j,9) = rh_i' * pfq33 * rh_j;
    end

    % lsa
    x = sub_factorization_lsa(A,g);
    %x = lsqr(A,g);

    % form new Q0
    dQ = zeros(3,3);
    dQ(1,:) = x(1:3)';
    dQ(2,:) = x(4:6)';
    dQ(3,:) = x(7:9)';
    Q0 = Q0 - dQ;
end

Q = Q0;

% least square method by pseudoinverse
function xH = sub_factorization_lsa(A,b)
xH = pinv(A'*A) * A' * b;
xH = pinv(A) * b;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function toolpaths=sub_add_path
current_dir=pwd;
toolpaths{1}=strcat(current_dir,'\cv_toolbox1');
toolpaths{2}=strcat(current_dir,'\ep_toolbox');
toolpaths{3}=strcat(current_dir,'\ar_toolbox');
nn=length(toolpaths);
for n=1:nn
    path(path,toolpaths{n})
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sub_remove_path(toolpaths)
nn=length(toolpaths);
for n=1:nn
    rmpath(toolpaths{n})
end
return
