% factorization_ct
% description: The main function for factorization method
% 
% Inputs:
% rpy_ix: x coordinate of feature points in images [no. of feature, no. of images] = size(rpy_ix)
% rpy_iy: y coordinate of feature points in images
% flen: focal length in no. of pixels
% init_center: initial center point of the 3D model
% noit: no. of iteration
% 
% Outputs:
% P: found model
% RT_sol: Translation [1:3] and Rotation angle [4:6]
% error_trucco_all_time: (not implemented)
% mse_error_model_pixels2_loop: (not implemented)
function [P,RT_sol,error_trucco_all_time,mse_error_model_pixels2_loop]...
    = factorization_ct(rpy_ix,rpy_iy,flen,init_center,noit)

[nf,Nimg] = size(rpy_ix);
RT_sol = zeros(6,Nimg);
% find X and Y for factorization W
% clear
% load foo
% [NN,TT]=size(rpy_ix)
%     for j=1:TT
%         v = find(~isnan(rpy_ix(:,j)));         %find valid points
%         iix(v,j)=rpy_ix(v,j);
%         iiy(v,j)=rpy_iy(v,j);
%     end
%

X = iix' * init_center(3) / flen;
Y = iiy' * init_center(3) / flen;
RT_sol(1,:)=mean(X'); %translation in  x
RT_sol(2,:)=mean(Y'); %translation in  y
pause(0)
% invoke factorization method
[R,S] = factorization(X,Y,noit);

% find the Direction of first Cam (respect to object centroid)
oi = R(1,:)';
oj = R(Nimg+1,:)';
ok=cross(oi,oj);
firstCamDirection = [oi oj ok]';

P = firstCamDirection*S;

% Rotation and translation (redundant for original factorization)
%RT_sol = zeros(6,Nimg);
RT_sol(1:3,1) = init_center;
for i=1:Nimg
    oi = R(i,:)';
    oj = R(Nimg+i,:)';
    ok = cross(oi,oj);
    rotation_matrix = inv(firstCamDirection) * [oi oj ok]';
    RT_sol(4:6,i) = rpyAng(rotation_matrix);
end
% 
% figure(561)
% clf
% hold on
% subplot(3,1,1),plot(RT_sol(4,:)*180/pi,'r-');ylabel('an1');
% subplot(3,1,2),plot(RT_sol(5,:)*180/pi,'r-');ylabel('an2');
% subplot(3,1,3),plot(RT_sol(6,:)*180/pi,'r-');ylabel('an3');
% xlabel('orginal factorization method , rotation angles vs time')
% 
% figure(562)
% clf
% hold on
% subplot(3,1,1),plot(RT_sol(1,:),'r-');ylabel('trans-X');
% subplot(3,1,2),plot(RT_sol(2,:),'r-');ylabel('trans-Y');
% subplot(3,1,3),plot(RT_sol(3,:),'r-');ylabel('trans-Z');
% xlabel('orginal factorization method , translation angles vs time')
% 
% 

error_trucco_all_time = zeros(noit,1);
mse_error_model_pixels2_loop = zeros(noit,1);
return


% factorization (Original FM, using Newton's method and least square method)
% X,Y
%
% R: camera reference geometry
% S: position of 3D points with respect to centriod of the points
function [R,S] = factorization(X,Y,noit)

% N: No of images
% n: No of feature pts
[N,n] = size(X);

% generate ~W
W = zeros(2*N,n);
W(1:N,:) = X;
W(N+1:2*N,:) = Y;
s_W = (sum(W'))' / n;
for i = 1:n
    W(:,i) = W(:,i) - s_W;
end

% svd
[U,D,V] = svds(W,3);
D2 = sqrtm(D);
RH = U*D2;
SH = D2*V';

Q = sub_factorization_newton(RH,noit);

R = RH * Q;
S = inv(Q) * SH;

function Q = sub_factorization_newton(RH,noit)
Q0 = [1 0 0;0 1 0;0 0 1];   %initial guess

[N,temp] = size(RH);
N = N / 2;
g = zeros(3*N,1);
rh_i = zeros(3,1);
rh_j = zeros(3,1);

for i = 1:noit
    % df = f' - f = g
    QQt = Q0 * Q0';
    for j = 1:N
        rh_i = (RH(j,:))';
        rh_j = (RH(j+N,:))';
        g(3*j-2) = rh_i' * QQt * rh_i - 1;
        g(3*j-1) = rh_j' * QQt * rh_j - 1;
        g(3*j) = rh_i' * QQt * rh_j;
    end
    
    % partial derivatives
    pfq11 = [2*Q0(1,1) Q0(2,1) Q0(3,1); Q0(2,1) 0 0; Q0(3,1) 0 0];
    pfq12 = [2*Q0(1,2) Q0(2,2) Q0(3,2); Q0(2,2) 0 0; Q0(3,2) 0 0];
    pfq13 = [2*Q0(1,3) Q0(2,3) Q0(3,3); Q0(2,3) 0 0; Q0(3,3) 0 0];
    pfq21 = [0 Q0(1,1) 0; Q0(1,1) 2*Q0(2,1) Q0(3,1); 0 Q0(3,1) 0];
    pfq22 = [0 Q0(1,2) 0; Q0(1,2) 2*Q0(2,2) Q0(3,2); 0 Q0(3,2) 0];
    pfq23 = [0 Q0(1,3) 0; Q0(1,3) 2*Q0(2,3) Q0(3,3); 0 Q0(3,3) 0];
    pfq31 = [0 0 Q0(1,1); 0 0 Q0(2,1); Q0(1,1) Q0(2,1) 2*Q0(3,1)];
    pfq32 = [0 0 Q0(1,2); 0 0 Q0(2,2); Q0(1,2) Q0(2,2) 2*Q0(3,2)];
    pfq33 = [0 0 Q0(1,3); 0 0 Q0(2,3); Q0(1,3) Q0(2,3) 2*Q0(3,3)];
    
    % Ax = b, A
    A = zeros(3*N,9);
    for j = 1:N
        rh_i = (RH(j,:))';
        rh_j = (RH(j+N,:))';

        A(3*j-2,1) = rh_i' * pfq11 * rh_i;
        A(3*j-2,2) = rh_i' * pfq12 * rh_i;
        A(3*j-2,3) = rh_i' * pfq13 * rh_i;
        A(3*j-2,4) = rh_i' * pfq21 * rh_i;
        A(3*j-2,5) = rh_i' * pfq22 * rh_i;
        A(3*j-2,6) = rh_i' * pfq23 * rh_i;
        A(3*j-2,7) = rh_i' * pfq31 * rh_i;
        A(3*j-2,8) = rh_i' * pfq32 * rh_i;
        A(3*j-2,9) = rh_i' * pfq33 * rh_i;

        A(3*j-1,1) = rh_j' * pfq11 * rh_j;
        A(3*j-1,2) = rh_j' * pfq12 * rh_j;
        A(3*j-1,3) = rh_j' * pfq13 * rh_j;
        A(3*j-1,4) = rh_j' * pfq21 * rh_j;
        A(3*j-1,5) = rh_j' * pfq22 * rh_j;
        A(3*j-1,6) = rh_j' * pfq23 * rh_j;
        A(3*j-1,7) = rh_j' * pfq31 * rh_j;
        A(3*j-1,8) = rh_j' * pfq32 * rh_j;
        A(3*j-1,9) = rh_j' * pfq33 * rh_j;

        A(3*j,1) = rh_i' * pfq11 * rh_j;
        A(3*j,2) = rh_i' * pfq12 * rh_j;
        A(3*j,3) = rh_i' * pfq13 * rh_j;
        A(3*j,4) = rh_i' * pfq21 * rh_j;
        A(3*j,5) = rh_i' * pfq22 * rh_j;
        A(3*j,6) = rh_i' * pfq23 * rh_j;
        A(3*j,7) = rh_i' * pfq31 * rh_j;
        A(3*j,8) = rh_i' * pfq32 * rh_j;
        A(3*j,9) = rh_i' * pfq33 * rh_j;
    end

    % lsa
    x = sub_factorization_lsa(A,g);
    %x = lsqr(A,g);
    
    % form new Q0
    dQ = zeros(3,3);
    dQ(1,:) = x(1:3)';
    dQ(2,:) = x(4:6)';
    dQ(3,:) = x(7:9)';
    Q0 = Q0 - dQ;
end
Q = Q0;
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% least square method by pseudoinverse
function xH = sub_factorization_lsa(A,b)
xH = pinv(A'*A) * A' * b;
xH = pinv(A) * b;
return
