%lowe321: same as 320, but treat invalid point in V as 0 
%input format:
%(1) xycell = K*2 cell array,  K=number of cameras, 
%    e.g. xycell{1,1}=ix of camera 1, xycell{1,2}=iy of camera 1
%(2) RK = 3*3*K,   K=1:K cameras, contains the rotational matrix between cameras
%    RK(:,:,1)=eye(3)
%
%(3) (Can be omitted) modelcell = K cell array, e.g. modelcell{1}=model used for camera 1
%
%Assumption
%Linear motion for the first 10 frames
%

function [model,rt,er,et,ix,iy]=lowe320()

%load('syn_dat');
load('real_dat');

NCamera=2;
automodel=true;     %if true, then the first 10 frames will be used to calibrate system
autoangle=false;   %to find intercamera angle
debug=false;        %reload data in debugfile
plotflag=false;
LM=false;            %levenberg-marquart
nloop=1;            %loop for each complete pose estimation
iter=5;             %iteration for each pose estimation

%RK(:,:,2)=rpymat([0;(85/90)*pi/2;0]);   %5 deg error in calibration

% kk=2;         %for debugging
% rt=rt(:,:,:,kk);
% rtref=rt;
% modelcell=modelcell(kk);
% modelref=modelcell;
% xycell=xycell(kk,:);
% RK=RK(:,:,kk);

%T=size(rt,3);   %T=number of frames in sequence
%[foo,T]=size(xycell{1,1});
[foo,T]=size(xycell{1,1})
T=T-1
K=2

%K=size(rt,4);   %K=number of cameras
if K<NCamera
    error('NCamera is too large');
end
K=NCamera;
xycell=xycell(1:K,:);
RK=RK(:,1:3,1:K);

if exist('rt')
    rtref=rt(:,:,:,1:K);
else
    rtref=[];
end
rt=zeros(3,4,T,K);          %clear everything for rt
for k=1:K
    %rt(1:3,1:3,1,k)=eye(3);
    rt(1:3,1:3,1,k)=RK(:,:,k);
end

if exist('modelcell')
    modelref=modelcell(1:K);    %save modelcell for comparison
else
    modelref=[];
end
modelcell=cell(K,1);        %clear modelcell to null

if ~exist('zinit')
    zinit=1;
end
if ~exist('flen')
    flen=1;
end



xycell          %xycell contains all the x,y locations of features

% flen(1)=1.1;
% flen(2)=0.9;
flen(1)=1;
flen(2)=1;
for k=1:K       %normalize x,y to flen=1
    x=xycell{k,1};
    xycell{k,1}=x/flen(k);
    y=xycell{k,2};
    xycell{k,2}=y/flen(k);
    xycell{k,3}=~isnan(x);
end
flen=1;

Vcell=cell(K,1);        %Vcell contains the projection matrix for model reconstruction

if debug~=true
    for k=1:K
        Vcell{k} = init_project_matrix(xycell{k,1},xycell{k,2},flen);
    end
end

if automodel==true & debug~=true      
    Tmax=10;
    for k=1:K
        for t=1:Tmax
            rt(1:3,1:3,t,k)=RK(1:3,1:3,k);      %autoangle not used
        end
    end
    [modelcell, rt]=init_klinearmodel(Vcell,Tmax,rt,xycell);
    plot_model(modelcell);
else
    Tmax=1;
    for k=1:K
        m=init_planarmodel(xycell{k,1},xycell{k,2},zinit,flen);
        m(1:3,:)=RK(:,:,k)'*m(1:3,:);
        modelcell{k}=m;
    end
end

if K>1          %if multicamera, compute intercam angle
    if autoangle==true      %autocal
        d1=squeeze(rt(1:3,4,1:Tmax,1));
        for k=2:K
            dk=squeeze(rt(1:3,4,1:Tmax,k));
            RK(1:3,1:3,k)=d2r(d1,dk);
        end
    end
end

%rt=rtref;
if debug==true              %reload to prev condition
    load debugfile
    debug=true;
end

%modelcell=modelref;     %for DEBUGGING, use true model

for loop=1:nloop
    if debug==true
        t1=t;
    elseif loop==1
        t1=Tmax+1;
    else
        t1=2;
    end
    for t=t1:T
        pause(0)
        t
        if t==100
            debug=true;
            save debugfile 
        end
        
        
        if loop==1 & t>Tmax      %init pose of frame t
            for k=1:K
                rt(:,:,t,k)=rt(:,:,t-1,k);
            end
            b2=t;
            b1=1;
        else loop==1 & t<=Tmax        %for debug, normally t>Tmax
            b1=1; b2=Tmax;
        end
        
        for n = 1:iter  
            pause(0)
            
            rt=update_rt(flen, modelcell, xycell, rt, t, RK, LM, rtref);        %estimate pose
            %[rtref(:,:,t,1) rt(:,:,t,1)]
            
            modelcell = update_model(flen, modelcell, xycell, Vcell, rt, b1, b2, t);        %estimate model
            
            if plotflag==true
                %print_graph(Config, rt, rtref, Angles, T);
                plot_model(modelcell);        %plot output model for checking, can be removed
            end
        end
        
        if ~isempty(rtref)
            for k=1:K
                [rtref(:,:,t,k) rt(:,:,t,k)]
            end
        end
        
        for tt=1:t
            [rtref(:,:,tt,1) rt(:,:,tt,1)]
        end
        for tt=1:t
            [rtref(:,:,tt,2) rt(:,:,tt,2)]
        end
    end
end

err=total_error(flen, modelcell, xycell, rt, 1, T)     %sum all error

save loweout
xycell                  %to show the number of points used
plot_model(modelcell);
if automodel==true
    print_graph(rt, rtref, Angles, Tmax);
else
    print_graph(rt, rtref, Angles, T);
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%              fucntions                                 %%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [modelcell,rt]=init_klinearmodel(Vcell,Tmax,rt,xycell)
%init model and pose based on first Tmax frames
%make sure each model point has at least 3 observations, otherwise can't reconstruct
K=size(Vcell,1);
V=Vcell{1};
T=size(V,4);
if Tmax>T
    error('error (init_linearmodel): number of frames is less than Tmax');
end
modelcell=cell(K,1);
Ncell=cell(K,1);
wcell=cell(K,1);
Zero=zeros(3);
Scell=cell(K,1);
Rinvcell=cell(K,1);
CScell=cell(K,1);
Msize=0;
CS=zeros(3,3,Tmax);     %sum the column's S entries
for k=1:K
    V=Vcell{k};
    [t1,t2,N,T]=size(V);
    Ncell{k}=N;
    p=xycell{k,1};
    %p=squeeze(V(1,1,:,:));  %use p to screen NaN points
    [I,J]=find(~isnan(p));
    Sparse=sparse(I,J,ones(size(I)));
    sumSparse=sum(Sparse(:,1:Tmax),2);
    w=find(sumSparse==Tmax);
    NN=length(w);
    Msize=Msize+NN;
    S=zeros(3*NN,3*Tmax);
    RS=zeros(3,3,NN);        %sum the row's S entries
    Rinv=zeros(3*NN,3*NN);
    for t=1:Tmax
        for n=1:NN
            nn=3*(n-1)+1;
            tt=3*(t-1)+1;
%             if isnan(V(1,1,w(n),t))        %invalid point, treat as zero
%                 AA=Zero;
%             else
                VR=V(:,:,w(n),t)*rt(1:3,1:3,t,k);
                AA=VR'*VR;
%             end
            S(nn:nn+2,tt:tt+2)=AA;
            RS(:,:,n)=RS(:,:,n)+AA;
            CS(:,:,t)=CS(:,:,t)+AA;
        end
    end
    for n=1:NN
        nn=3*(n-1)+1;
        Rinv(nn:nn+2,nn:nn+2)=inv(RS(:,:,n));  %can try replacing inv by pinv
    end
    Scell{k}=S;
    Rinvcell{k}=Rinv;
    CScell{k}=CS;
    wcell{k}=w;
    Msizecell{k}=NN;
end

% for k=1:K
%     V=Vcell{k};
%     [t1,t2,N,T]=size(V);
%     Ncell{k}=N;
%     p=xycell{k,1};
%     %p=squeeze(V(1,1,:,:));  %use p to screen NaN points
%     [I,J]=find(~isnan(p));
%     Sparse=sparse(I,J,ones(size(I)));
%     sumSparse=sum(Sparse(:,1:Tmax),2);
%     w=find(sumSparse==Tmax);
%     NN=length(w);
%     Msize=Msize+NN;
%     S=zeros(3*NN,3*Tmax);
%     RS=zeros(3,3,NN);        %sum the row's S entries
%     Rinv=zeros(3*NN,3*NN);
%     for t=1:Tmax
%         for n=1:NN
%             nn=3*(n-1)+1;
%             tt=3*(t-1)+1;
%             if isnan(V(1,1,w(n),t))        %invalid point, treat as zero
%                 AA=Zero;
%             else
%                 VR=V(:,:,w(n),t)*rt(1:3,1:3,t,k);
%                 AA=VR'*VR;
%             end
%             S(nn:nn+2,tt:tt+2)=AA;
%             RS(:,:,n)=RS(:,:,n)+AA;
%             CS(:,:,t)=CS(:,:,t)+AA;
%         end
%     end
%     for n=1:NN
%         nn=3*(n-1)+1;
%         Rinv(nn:nn+2,nn:nn+2)=inv(RS(:,:,n));  %can try replacing inv by pinv
%     end
%     Scell{k}=S;
%     Rinvcell{k}=Rinv;
%     CScell{k}=CS;
%     wcell{k}=w;
%     Msizecell{k}=NN;
% end

G=zeros(3*Msize,3*Tmax);
A=zeros(3*Msize,3*Msize);
C=zeros(3*Tmax,3*Tmax);
N1=1;
for k=1:K
    N2=N1+3*Msizecell{k}-1;
    S=Scell{k};
    G(N1:N2,:)=S;
    Rinv=Rinvcell{k};
    A(N1:N2,N1:N2)=Rinv;
    N1=N2+1;
end

for t=1:Tmax
    tt=3*(t-1)+1;
    C(tt:tt+2,tt:tt+2)=CS(:,:,t);
end

B=G'*A*G-C;
B=B(:,4:end);
[u,s,v]=svd(B);
D=[[0;0;0]; v(:,end)];
m=A*G*D;
m=reshape(m,3,[]);
N=Msizecell{1};
v=find(m(3,1:N)<0);
if length(v) > (N/2)
    m=-m;
    D=-D;
end

N1=1;
for k=1:K
    N=Ncell{k};
    NN=Msizecell{k};
    N2=N1+NN-1;
    w=wcell{k};
    model=[repmat(NaN,3,N); zeros(1,N)];
    model(1:3,w)=m(:,N1:N2);
    model(4,w)=ones(1,NN);
    modelcell{k}=model;
    N1=N2+1;
    rt(:,4,1:Tmax,k)=reshape(D,3,[]);
end
return

function [modelcell,rt]=init_linearmodel(Vcell,Tmax,rt)
%init model and pose based on first Tmax frames
%make sure each model point has at least 3 observations, otherwise can't reconstruct
K=size(Vcell,1);
V=Vcell{1};
T=size(V,4);
if Tmax>T
    error('error (init_linearmodel): number of frames is less than Tmax');
end
modelcell=cell(K,1);
D=zeros(3*Tmax,K);
Zero=zeros(3);
for k=1:K
    V=Vcell{k};
    [t1,t2,N,T]=size(V);
    p=squeeze(V(1,1,:,:));  %use p to screen NaN points
    [I,J]=find(~isnan(p));
    Sparse=sparse(I,J,ones(size(I)));
    sumSparse=sum(Sparse(:,1:Tmax),2);
    w=find(sumSparse==Tmax);
    NN=length(w);
    S=zeros(3*NN,3*Tmax);
    RS=zeros(3,3,NN);        %sum the row's S entries
    CS=zeros(3,3,Tmax);     %sum the column's S entries
    for n=1:NN
        for t=1:Tmax
            nn=3*(n-1)+1;
            tt=3*(t-1)+1;
            if isnan(V(1,1,w(n),t))        %invalid point, treat as zero
                AA=Zero;
            else
                VR=V(:,:,w(n),t)*rt(1:3,1:3,t,k);
                AA=VR'*VR;
            end
            S(nn:nn+2,tt:tt+2)=AA;
            RS(:,:,n)=RS(:,:,n)+AA;
            CS(:,:,t)=CS(:,:,t)+AA;
        end
    end
    
    Rinv=zeros(3*NN,3*NN);
    C=zeros(3*Tmax,3*Tmax);
    for n=1:NN
        nn=3*(n-1)+1;
        Rinv(nn:nn+2,nn:nn+2)=inv(RS(:,:,n));  %can try replacing inv by pinv
    end
    for t=1:Tmax
        tt=3*(t-1)+1;
        C(tt:tt+2,tt:tt+2)=CS(:,:,t);
    end
    
    A=S'*Rinv*S-C;
    A=A(:,4:end);
    [u,s,v]=svd(A);
    D(:,k)=[[0;0;0]; v(:,end)];
    m=Rinv*S*D(:,k);
    m=reshape(m,3,[]);
    mm=rt(1:3,1:3,1,k)*m;
    v=find(mm(3,:)<0);
    if length(v) > (NN/2)
        m=-m;
        D(:,k)=-D(:,k);
    end
    model=[repmat(NaN,3,N); zeros(1,N)];
    model(1:3,w)=m;
    model(4,w)=ones(1,NN);
    modelcell{k}=model;
    
    rt(:,4,1:Tmax,k)=reshape(D(:,k),3,[]);
end
return

function model=init_planarmodel(x,y,zinit,flen)
[N,T]=size(x);
model=zeros(4,N);
v=find(~isnan(x(:,1)));
model(:,v) = [(zinit/flen)*x(v,1)'; (zinit/flen)*y(v,1)'; zinit*ones(size(v))'; ones(size(v))']; %planar model
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function e=column_err(flen,model,ix,iy,rt)
%return err, a column vector of max size 3*N*T
[N,T]=size(ix);
if T~=1
    error('column_err: ix, iy must be a column vector');
end
X = rt(1:3,1:3)*(model(1:3,:)-repmat(rt(1:3,4),1,N));
x = flen.*X(1,:)./X(3,:);
y = flen.*X(2,:)./X(3,:);
ex = ix - x';
ey = iy - y';
e = [ex; ey];
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function err=project_err(flen,model,ix,iy,rt,b1,b2)
%return err, a column vector of max size 3*N*T
T=size(rt,3);
err=[];
for t=b1:b2
    v = find(~isnan(ix(:,t)));         %find valid points that is not the start point
    w = find(model(4,:)==1);        %find uninitialised points
    v = intersect(v,w);               %get common good points
    column_err(flen,model(:,v),ix(v,t),iy(v,t),rt(:,:,t));    
    err = [err; column_err(flen,model(:,v),ix(v,t),iy(v,t),rt(:,:,t))];
end
return

function J=JacobRT(flen, model, rt, rk)
%cam model _ P=R(M-T), rt=3*4
N = size(model,2);    
P=rt(1:3,1:3)*(model(1:3,:)-repmat(rt(1:3,4),1,N));
X=P(1,:)';
Y=P(2,:)';
Z=P(3,:)';
PK=rk*P;
XK=PK(1,:)';
YK=PK(2,:)';
ZK=PK(3,:)';

J=zeros(2*N,6);

iZ = 1./ZK;
iZZ = iZ.^2;
XZZ = XK.*iZZ;
YZZ = YK.*iZZ;
zero=zeros(N,1);
dx = [iZ zero -XZZ]; 
dy = [zero iZ -YZZ];

for i=1:N
    %dX = [0 Z(i) -Y(i); -Z(i) 0 X(i); Y(i) -X(i) 0];
    dX = [0 -Z(i) Y(i); Z(i) 0 -X(i); -Y(i) X(i) 0];
    J(i,:)  = dx(i,:)*[ -rk*rt(1:3,1:3) rk*dX ];
    J(N+i,:)= dy(i,:)*[ -rk*rt(1:3,1:3) rk*dX ];
end

J=flen*J;
return

function J=JacobT(flen, model, rt, rk)
%cam model _ P=R(M-T), rt=3*4
rk=rk(1:3,1:3);
N = size(model,2);    
J=zeros(2*N,3);     %J=[Jx; Jy]
P=rt(1:3,1:3)*(model(1:3,:)-repmat(rt(1:3,4),1,N));
X=P(1,:)';
Y=P(2,:)';
Z=P(3,:)';

iZ = 1./Z;
iZZ = iZ.^2;
XZZ = X.*iZZ;
YZZ = Y.*iZZ;
zero=zeros(N,1);
dx = [iZ zero -XZZ]; 
dy = [zero iZ -YZZ];

for i=1:N
    %dX = [0 Z(i) -Y(i); -Z(i) 0 X(i); Y(i) -X(i) 0];
    J(i,:)=dx(i,:)*[ -rt(1:3,1:3)*rk ];
    J(N+i,:)=dy(i,:)*[ -rt(1:3,1:3)*rk ];
end

J=flen*J;
return


function rt=update_rt(flen, modelcell, xycell, rt, j, RK, LM, rtref) %rt is of size 3x4
%estimate the pose of jth frame
K=size(rt,4);
trt=zeros(3,4,K);
if LM==true         %use levenberg marquart
    lambda=100;
    eye_mat=eye(6,6);
    nloop=20;
else
    nloop=5;
end
for loop=1:nloop
    err=[]; J=[]; W=[];
    for k=1:K
        ix=xycell{k,1}; 
        v=find(~isnan(ix(:,j)));
        model=modelcell{k};
        w=find(model(4,:)==1);
        v=intersect(v,w);
        iy=xycell{k,2};
        e=column_err(flen, model(:,v), ix(v,j), iy(v,j), rt(:,:,j,k));
        err=[err; e];
        J = [J; JacobRT(flen, model(:,v), rt(:,:,j,1), RK(:,:,k))];
    end
    err1=mmse(err);
    %     %limit the outlier to no greater than 2*sigma
    %     N=size(err,1);
    %     sigma=std(err);
    %     sigma=2*sigma;
    %     abserr=abs(err);
    %     v=find(abserr>sigma);
    %     s = sigma./abserr(v);
    %     W=ones(N,1);
    %     W(v)=s;
    %     W=diag(W);
    %     err=W'*err;
    %     J=W'*J;
    %     cond(J)
    
    %err1=norm(err)
    if LM==true
        err = J'*err;  
        JJ = J'*J;
        J = JJ+JJ.*(lambda*eye_mat);    %add the Levenberg-Marquardt term    
    end
    
    %     JJ=J'*J;
    %     [u,s,v]=svd(JJ);
    %     diags=diag(s)'
    
    pause(0)
    delta = J\err;              %delta=3T+3T
    dT = delta(1:3);
    dR = delta(4:6);
    trt(1:3,4,1) = rt(1:3,4,j,1)+dT;
    trt(1:3,1:3,1) = rpymat(dR)*rt(1:3,1:3,j,1);
    [trt(:,:,1) rtref(:,:,j,1)]
%     for k=2:K
%         trt(1:3,4,k) = trt(1:3,4,1);
%         trt(1:3,1:3,k) = RK(:,:,k)*trt(1:3,1:3,1);
%         [trt(:,:,k) rtref(:,:,j,k)]
%     end
    
%     for k=2:K
%         trt(1:3,4,k) = trt(1:3,4,1);
%         trt(1:3,1:3,k) = rpymat(dR)*rt(1:3,1:3,j,k);
%         [trt(:,:,k) rtref(:,:,j,k)]
%     end
    
    W=[]; err=[];  %clear W
    for k=1:K
        ix=xycell{k,1}; 
        v=find(~isnan(ix(:,j)));
        model=modelcell{k};
        w=find(model(4,:)==1);
        v=intersect(v,w);
        iy=xycell{k,2};
        trt(:,:,k)=trt(:,:,1);
        trt(1:3,1:3,k)=RK(:,:,k)*trt(1:3,1:3,1);
        e=column_err(flen, model(:,v), ix(v,j), iy(v,j), trt(:,:,k));
        err=[err; e];
    end
    err2=mmse(err);
    
    [err1 err2]
    if err1>=err2    %smaller error, good
        rt(:,:,j,:)=trt;
        if LM==true
            lambda = lambda/2;
        end
    else            %err1<err2, bad
        if LM==true
            lambda = lambda*2;
        else    %start to diverge, stop
            break
        end
    end
end
return

function rt=update_disp(flen, modelcell, xycell, rt, j) %rt is of size 3x4
K=size(rt,4);
lambda=20;
eye_mat=eye(3,3);
trt=rt;
for loop=1:5
    err=[]; J=[]; W=[];
    for k=1:K
        ix=xycell{k,1}; 
        v=find(~isnan(ix(:,j)));
        model=modelcell{k};
        w=find(model(4,:)==1);
        v=intersect(v,w);
        iy=xycell{k,2}; 
        e=column_err(flen, model(:,v), ix(v,j), iy(v,j), rt(:,:,j,k));
        err=[err; e];
        J = [J; JacobT(flen, model(:,v), rt(:,:,j,k), rt(:,:,1,k))];
    end
    err1=mmse(err);
    %limit the outlier to no greater than 2*sigma
    N=size(err,1);
    sigma=std(err);
    sigma=2*sigma;
    abserr=abs(err);
    v=find(abserr>sigma);
    s = sigma./abserr(v);
    W=ones(N,1);
    W(v)=s;
    W=diag(W);
    err=W'*err;
    J=W'*J;
    cond(J)
    
    err1=mmse(err);
    [j K]
    size(J)
    Jerr = J'*err;  
    JJ = J'*J;
    LM = lambda*eye_mat;
    JM = JJ+JJ.*LM;    %add the Levenberg-Marquardt term    
    delta=JJ\Jerr;    
    
    pause(0)
    %delta = J\err;             %delta=3T+3T
    delta = JM\Jerr;            %delta=3T+3T
    dT = delta(1:3);
    for k=1:K
        %         rt(1:3,4,j,k) = rt(1:3,4,j,k)+dT;
        trt(1:3,4,j,k) = rt(1:3,4,j,k)+dT;
    end
    
    %compute error
    W=[]; err=[];  %clear W
    for k=1:K
        ix=xycell{k,1}; 
        v=find(~isnan(ix(:,j)));
        model=modelcell{k};
        w=find(model(4,:)==1);
        v=intersect(v,w);
        iy=xycell{k,2};
        %e=column_err(flen, model(:,v), ix(v,j), iy(v,j), rt(:,:,j,k));
        e=column_err(flen, model(:,v), ix(v,j), iy(v,j), trt(:,:,j,k));
        err=[err; e];
    end
    err2=mmse(err);
    [err1 err2]
    %     if (err1-err2)/err1<0.01
    %         break
    %     end
    if err2 <= err1
        lambda = lambda/2;
        rt=trt;
    else        %lambda too big, rt is not changed
        lambda = lambda*2;
    end
end
return

% Author: Rodrigo Carceroni
% Disclaimer: This code comes with no guarantee at all and its author
%   is not liable for any damage that its utilization may cause.

function R = rpymat (angs)
% Return the 3x3 rotation matrix described by a set of Roll, Pitch and Yaw angles.

c = cos(angs(1));
s = sin(angs(1));
%Rx=[1 0 0; 0 c -s; 0 s c];
Rx=[1 0 0; 0 c s; 0 -s c];      %reverse rotation for camera  center
c = cos(angs(2));
s = sin(angs(2));
%Ry=[c 0 s; 0 1 0; -s 0 c];
Ry=[c 0 -s; 0 1 0; s 0 c];
c = cos(angs(3));
s = sin(angs(3));
%Rz=[c -s 0; s c 0; 0 0 1];
Rz=[c s 0; -s c 0; 0 0 1];

R=Rz*Ry*Rx;

% cosA = cos (angs(3));
% sinA = sin (angs(3));
% cosB = cos (angs(2));
% sinB = sin (angs(2));
% cosC = cos (angs(1));
% sinC = sin (angs(1));
% 
% cosAsinB = cosA .* sinB;
% sinAsinB = sinA .* sinB;
% 
% R = [ cosA.*cosB  cosAsinB.*sinC-sinA.*cosC  cosAsinB.*cosC+sinA.*sinC ;
%       sinA.*cosB  sinAsinB.*sinC+cosA.*cosC  sinAsinB.*cosC-cosA.*sinC ;
%         -sinB            cosB.*sinC                 cosB.*cosC         ];
return

function angs = rpyang (R)

%R = [          cz*cy, sz*cx+cz*sy*sx, sz*sx-cz*sy*cx]
%    [         -sz*cy, cz*cx-sz*sy*sx, cz*sx+sz*sy*cx]
%    [             sy,         -cy*sx,          cy*cx]
 
sy = R(3,1);
cy = sqrt( R(1,1).* R(1,1) + R(2,1).* R(2,1) );
if abs(cy) > 1e-15
  sz = -R(2,1) ./ cy;
  cz =  R(1,1) ./ cy;
  sx = -R(3,2) ./ cy;
  cx =  R(3,3) ./ cy;
  angs = [atan2(sx,cx); atan2(sy,cy); atan2(sz,cz)];
else   %if cy=0, thus sy=1, ist col = [0;0;1], 3rd row = [1 0 0], so angs(2)=pi/2, angs(3)=0
  if sy > 0  %cz=1, sz=0
        sx = (R(1,2) + R(2,3)) ./ 2;
        cx = (R(2,2) - R(1,3)) ./ 2;
        angs = [atan2(sx,cx); -pi./2; 0];
    else %sy < 0
        sx = ( -R(1,2) + R(2,3)) ./ 2;
        cx = ( R(2,2) + R(1,3)) ./ 2;
        angs = [atan2(sx,cx); pi./2; 0];
    end
  end
end
return

function model=update_model_point(ix,iy,i,b1,b2,V,rt)      %find model pt from pose, for R(M-T)
model=zeros(4,1);
Asum=0; Bsum=0;

for j=b1:b2
    if ~isnan(ix(i,j))
        A = V(:,:,i,j)*rt(:,1:3,j);  %v(j)=frame number (i.e. between 1:T)
        AA=A'*A;
        Asum = Asum+AA;
        Bsum = Bsum+AA*rt(1:3,4,j);
    end
end

if rcond(Asum)<100*eps     %if asum is closed to singular
    model(4)=0;             %model(4) is already 0, just to make this more explicit
else
    model(1:3)=Asum\Bsum;
    model(4)=1;             %mark model point as initialized
end
return

function V = init_project_matrix(ix,iy,flen)
[N,T]=size(ix);
V = zeros(3,3,N,T);     %init the projection matrix for building the model
z3=zeros(3,3);
for i=1:N
    v = [ix(i,:); iy(i,:); flen*ones(1,T)];
    for j=1:T
        if ~isnan(ix(i,j))
            V(:,:,i,j)=eye(3)-v(:,j)*v(:,j)'/(v(:,j)'*v(:,j));
        else
            V(:,:,i,j)=z3;
        end
    end
end
return

function modelcell = update_model(flen, modelcell, xycell, Vcell, rt, b1, b2,t)
MIN_MODEL_LENGTH=10;
K=size(rt,4);
for k=1:K
    m=modelcell{k};
    ix=xycell{k,1};
    iy=xycell{k,2};
    valid=xycell{k,3};
    N=size(m,2);
    v=valid(:,b1:b2);
    sumv=sum(v,2);      %sum number of ones in each row
    for i=1:N           %can be improved by v=find(model(4,:)==1); for i=1:length(v) w=find(~isnan(ix(v(i),1:t));
        if valid(i,t)       %if ith point of current frame is valid, new info, update
            if sumv(i)>=MIN_MODEL_LENGTH
                m(:,i)=update_model_point(ix,iy,i,b1,b2,Vcell{k},rt(:,:,:,k));
            end
        end
    end
    modelcell{k}=m;
end
return

function plot_model(modelcell)
K=size(modelcell,1);
for k=1:K       
    figure(10*K+k)
    m=modelcell{k};
    v=find(m(4,:)==1);      %get valid model point
    scatter3(m(1,v),m(2,v),m(3,v));
    axis equal
    xlabel('x axis');
    ylabel('y axis');
    zlabel('z axis');
end
return

function mse=total_error(flen, modelcell, xycell, rt, b1, b2)
K=size(rt,4);
err=[];
for k=1:K
    err=[err; project_err(flen, modelcell{k}, xycell{k,1}, xycell{k,2}, rt, b1, b2)];
end
mse=mmse(err);    %get root mean square 
return

function mse=mmse(err)
mse=sqrt(2)*sqrt(mean(err.^2));    %get root mean square 
return

function print_graph(rt, rtref, Angles, Tmax)
K=size(rt,4);
T=size(Angles,2);
figure(K*100+99)              %plot displacement error
hold off
clf
disp=reshape(rt(:,4,:,1),3,[]);
ref=reshape(rtref(:,4,:),3,[]);

%s = sum(xy)/sum(y*y) by least square method, where x=sy, x is reference
if ~isempty(ref)        %if ref is not empty
    xy=disp(:,1:Tmax).*ref(:,1:Tmax);
    yy=disp(:,1:Tmax).*disp(:,1:Tmax);
    s=sum(sum(xy))/sum(sum(yy));
    disp=s*disp;
end
maxd=1.1*max(max(disp));
mind=min(min(disp));

subplot(3,1,1);  plot(disp(1,:),'ro:'); hold on
if ~isempty(ref)
    subplot(3,1,1); plot(ref(1,:),'b');
end
axis([1 T mind maxd]);

subplot(3,1,2); plot(disp(2,:),'ro:'); hold on
if ~isempty(ref)
    subplot(3,1,2); plot(ref(2,:),'b');
end
axis([1 T mind maxd]);

subplot(3,1,3); plot(disp(3,:),'ro:'); hold on
if ~isempty(ref)
    subplot(3,1,3); plot(ref(3,:),'b');
end
axis([1 T mind maxd]);

figure(K*100+98)              %plot angular error
clf
k=1; 
angles=zeros(3,T);
angles(:,1)=rpyang(rt(1:3,1:3,1,k));    %get first angles
for t=2:T
    %was angles(:,t)=rpyang(rt(1:3,1:3,t,k), but has problem when ang is large
    dR=rt(1:3,1:3,t,k)*rt(1:3,1:3,t-1,k)';
    ang=rpyang(dR);
    angles(:,t)=angles(:,t-1)+ang;   %to avoid multiple solutions in rpyang when ang is large
end

% maxd=1.1*max(max(Angles));
% mind=min(min(Angles));
maxd=1.1*max(max(Angles));
mind=-0.5;
subplot(3,1,1);  plot(angles(1,:),'ro:'); hold on
if exist('Angles')
    subplot(3,1,1); plot(Angles(1,:),'b');
end
axis([1 T mind maxd]);
subplot(3,1,2); plot(angles(2,:),'ro:'); hold on
if exist('Angles')
    subplot(3,1,2); plot(Angles(2,:),'b');
end
axis([1 T mind maxd]);
subplot(3,1,3); plot(angles(3,:),'ro:'); hold on
if exist('Angles')
    subplot(3,1,3); plot(Angles(3,:),'b');
end
axis([1 T mind maxd]);
return